<?php

declare(strict_types=1);

namespace App\Command;

use App\Helper\LoggerHelper;
use App\Jmcomic\Client;
use App\Jmcomic\ClientFactory;
use App\Model\Album;
use App\Model\Chapter;
use App\Model\ChapterImage;
use App\Transfer\ImageCenterDistributor;
use GuzzleHttp\Exception\GuzzleException;
use Hyperf\Command\Command as HyperfCommand;
use Hyperf\Command\Annotation\Command;
use Hyperf\DbConnection\Db;
use Hyperf\Di\Annotation\Inject;
use Psr\Container\ContainerInterface;
use Symfony\Component\Console\Helper\ProgressBar;

/**
 * @Command
 */
#[Command]
class ComicMonitorCommand extends HyperfCommand
{
    #[Inject]
    protected ClientFactory $clientFactory;

    #[Inject]
    protected ImageCenterDistributor $distributor;

    private Client $client;

    public function __construct()
    {
        parent::__construct('comics:monitor');
    }

    public function configure()
    {
        parent::configure();
        $this->setDescription('Hyperf Demo Command');
    }

    public function handle()
    {
        try {
            $this->client = $this->clientFactory->create();
            // all albums check update
            Album::query()->chunk(100, function ($albums) {
                foreach ($albums as $album) {
                    $this->handleAlbum($album);
                }
            });
        } catch (GuzzleException|\Throwable|\Exception $e) {
            LoggerHelper::printStackTrace($e);
        }
    }

    /**
     * @throws \Throwable
     * @throws GuzzleException
     * @throws \JsonException
     */
    private function handleAlbum(Album $album)
    {
        $this->line("album: {$album->name}", 'info');
        $payload = $this->client->album($album->source_id);
        // 比较tags 是否一致
        if ($album->tags !== $payload['tags']) {
            $album->tags = $payload['tags'];
            $album->save();
        }
        if ($album->chapters == count($payload['series'])) {
            // chapter not update
            return;
        }
        // album have chapter update
        $this->line('album have chapter update', 'info');

        // get chapter from web
        $webInfo = $this->client->albumWebInfo($album->source_id);
        $chapterList = $webInfo['chapterList'];
        $album->chapters = count($chapterList);
        $album->last_updated = $webInfo['updateDate'];
        $album->save();

        foreach ($chapterList as $idx => $chapter) {
            $this->handleChapter($album, $idx, $chapter);
        }
    }


    private function handleChapter(Album $album, int $sort, array $chapter)
    {
        // check chapter exist
        $exists = Chapter::query()->where([
            ['album_id', '=', $album->id],
            ['source_id', '=', $chapter['id']],
        ])->where('sort', $sort)->exists();
        if ($exists) {
            return;
        }
        // 章节应该跟图片一起保存成功
        Db::transaction(function () use ($sort, $album, $chapter) {
            $this->line("new chapter: {$chapter['chapterName']}", 'info');
            $payload = $this->client->chapter($chapter['id']);
            $chapterModel = new Chapter();
            $chapterModel->album_id = $album->id;
            $chapterModel->name = $chapter['chapterName'];
            $chapterModel->sort = $sort;
            $chapterModel->publish_at = $chapter['published'];
            $chapterModel->source_id = $chapter['id'];
            $chapterModel->images = count($payload['images']);
            $chapterModel->save();
            $this->line("chapter image count: " . $chapterModel->images, 'info');
            foreach ($payload['images'] as $file) {
                $image = new ChapterImage();
                $image->album_id = $album->id;
                $image->chapter_id = $chapterModel->id;
                $image->unique_id = $this->distributor->distribute([
                    'source_src' => "/media/photos/${chapter['id']}/{$file}",
                    'source_type' => ImageCenterDistributor::SOURCE_TYPE_CHAPTER_IMAGE,
                    'from_album_id' => $album->id,
                    'from_chapter_id' => $chapterModel->id,
                ]);
                $image->save();
            }
        });

    }
}
