<?php

declare(strict_types=1);
/**
 * This file is part of Hyperf.
 *
 * @link     https://www.hyperf.io
 * @document https://hyperf.wiki
 * @contact  group@hyperf.io
 * @license  https://github.com/hyperf/hyperf/blob/master/LICENSE
 */
namespace App\Transfer;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Hyperf\Config\Annotation\Value;
use Hyperf\Contract\StdoutLoggerInterface;
use Hyperf\Di\Annotation\Inject;
use Hyperf\Guzzle\ClientFactory;
use Psr\Http\Message\StreamInterface;
use Psr\SimpleCache\CacheInterface;
use Psr\SimpleCache\InvalidArgumentException;

class DouyuYBTransfer implements TransferInterface
{
    #[Inject]
    protected StdoutLoggerInterface $logger;

    #[Inject]
    protected CacheInterface $cache;

    #[Value('nacos_config.transfer.douyuyb.cookie')]
    protected string $cookie;

    private array $uploadParams = [];

    private Client $client;

    public function __construct(ClientFactory $clientFactory)
    {
        $this->client = $clientFactory->create();
    }

    /**
     * @throws GuzzleException
     * @throws \JsonException
     * @throws \Exception
     */
    public function upload(StreamInterface $stream, string $filename): TransferResult
    {
        $params = $this->getUploadParams();
        $multipart = [];
        foreach ($params as $key => $value) {
            $multipart[] = [
                'name' => $key,
                'contents' => $value,
            ];
        }
        $multipart[] = [
            'name' => 'file',
            'contents' => $stream,
            'filename' => $filename,
        ];

        $resp = $this->client->post('https://img.douyucdn.cn/upload', [
            'multipart' => $multipart,
        ]);
        $payload = json_decode($resp->getBody()->getContents(), true, flags: JSON_THROW_ON_ERROR);
        if ($payload['errorCode'] !== 0) {
            throw new \Exception('douyu yuba upload error reason:' . $payload['errorMessage']);
        }

        return new TransferResult($payload['baseUrl'] . $payload['uri']);
    }

    /**
     * @throws GuzzleException|InvalidArgumentException
     */
    private function getUploadParams(): array
    {
        // 尝试从缓存中获取
        if (empty($this->uploadParams) && $this->cache->has('douyu_yb_upload_params')) {
            $this->uploadParams = $this->cache->get('douyu_yb_upload_params');
        }

        // 如果还是为空或者过期了，重新获取
        if (empty($this->uploadParams) or $this->uploadParams['expired'] < time()) {
            $this->uploadParams = $this->preUpload($this->cookie);
            // 设置缓存
            $this->cache->set('douyu_yb_upload_params', $this->uploadParams, $this->uploadParams['expired'] - time());
        }
        $this->logger->debug('douyu yuba upload params:' . var_export($this->uploadParams, true));

        return $this->uploadParams;
    }

    /**
     * @throws GuzzleException
     * @throws \Exception
     */
    private function preUpload(string $cookie): array
    {
        $url = sprintf('https://yuba.douyu.com/wbapi/web/image/preRequest?source=&timestamp=%d', time());
        $response = $this->client->get($url, [
            'headers' => ['cookie' => $cookie],
        ]);
        $contents = $response->getBody()->getContents();
        if (empty($contents)) {
            throw new \Exception('douyu yuba preRequest response is empty');
        }
        $payload = json_decode($contents, true, flags: JSON_THROW_ON_ERROR);
        if ($payload['status_code'] !== 200) {
            throw new \Exception('douyu yuba preRequest response error' . "${payload['status_code']}: ${payload['message']}");
        }
        return $payload['data'];
    }
}
